@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2018 Stefan Pöschel
+    Copyright (C) 2015-2019 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,8 +46,10 @@ static void usage(const char* exe) {
 					"  -P <size>    Recording prebuffer size in seconds (default: %zu)\n"
 					"  -p           Output PCM to stdout instead of using SDL\n"
 					"  -u           Output untouched audio stream to stdout instead of using SDL\n"
+					"  -I           Don't catch up on stream after interruption\n"
 					"  -S           Initially disable slideshow\n"
 					"  -L           Enable loose behaviour (e.g. PAD conformance)\n"
+					"  -F           Disable dynamic FIC messages (e.g. dynamic PTY)\n"
 					"  file         Input file to be played (stdin, if not specified)\n",
 					DABLiveETISource::TYPE_DAB2ETI.c_str(),
 					DABLiveETISource::TYPE_ETI_CMDLINE.c_str(),
@@ -74,7 +76,7 @@ int main(int argc, char **argv) {
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:P:s:x:puSL")) != -1) {
+	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:P:s:x:puISLF")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
@@ -117,12 +119,18 @@ int main(int argc, char **argv) {
 		case 'u':
 			options.untouched_output = true;
 			break;
+		case 'I':
+			options.disable_int_catch_up = true;
+			break;
 		case 'S':
 			options.initially_disable_slideshow = true;
 			break;
 		case 'L':
 			options.loose = true;
 			break;
+		case 'F':
+			options.disable_dyn_fic_msgs = true;
+			break;
 		case '?':
 		default:
 			usage(argv[0]);
@@ -209,6 +217,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	initial_channel_appended = false;
 	rec_file = nullptr;
 	rec_duration_ms = 0;
+	rec_prebuffer_filled_ms = 0;
 
 	slideshow_window.set_transient_for(*this);
 
@@ -219,7 +228,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	pad_change_dynamic_label.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeDynamicLabelEmitted));
 	pad_change_slide.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeSlideEmitted));
 
-	eti_player = new ETIPlayer(options.pcm_output, options.untouched_output, this);
+	eti_player = new ETIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
 
 	if(!options.dab_live_source_binary.empty()) {
 		eti_source = nullptr;
@@ -228,11 +237,11 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 		eti_source_thread = std::thread(&ETISource::Main, eti_source);
 	}
 
-	fic_decoder = new FICDecoder(this);
+	fic_decoder = new FICDecoder(this, options.disable_dyn_fic_msgs);
 	pad_decoder = new PADDecoder(this, options.loose);
 
 	set_title("DABlin v" + std::string(DABLIN_VERSION));
-	set_default_icon_name("media-playback-start");
+	set_icon_name("media-playback-stop");
 
 	InitWidgets();
 
@@ -243,8 +252,9 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 		combo_channels.set_active(initial_channel_it);
 	initial_channel_appended = true;
 
-	ConnectKeyPressEventHandler(*this);
-	ConnectKeyPressEventHandler(slideshow_window);
+	// add window key press event handler
+	signal_key_press_event().connect(sigc::mem_fun(*this, &DABlinGTK::HandleKeyPressEvent));
+	add_events(Gdk::KEY_PRESS_MASK);
 
 	// add window config event handler (before default handler)
 	signal_configure_event().connect(sigc::mem_fun(*this, &DABlinGTK::HandleConfigureEvent), false);
@@ -391,10 +401,10 @@ void DABlinGTK::AddChannels() {
 			AddChannel(dab_channel, options.gain);
 	} else {
 		// add specific channels
-		string_vector_t channels = MiscTools::SplitString(options.displayed_channels, ',');
+		string_vector_t channels = StringTools::SplitString(options.displayed_channels, ',');
 		for(const std::string& channel : channels) {
 			int gain = options.gain;
-			string_vector_t parts = MiscTools::SplitString(channel, ':');
+			string_vector_t parts = StringTools::SplitString(channel, ':');
 			switch(parts.size()) {
 			case 2:
 				gain = strtol(parts[1].c_str(), nullptr, 0);
@@ -427,22 +437,28 @@ void DABlinGTK::AddChannel(const dab_channels_t::value_type& dab_channel, int ga
 void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 	// (re)set labels/tooltips/record button
 	if(!service.IsNone()) {
-		char sid_string[7];
-		snprintf(sid_string, sizeof(sid_string), "0x%04X", service.sid);
-
 		std::string charset_name;
-		Glib::ustring label = FICDecoder::ConvertLabelToUTF8(service.label, &charset_name);
+		std::string label = FICDecoder::ConvertLabelToUTF8(service.label, &charset_name);
 
 		set_title(label + " - DABlin");
-		frame_combo_services.set_tooltip_text(
-				"Short label: \"" + DeriveShortLabel(label, service.label.short_label_mask) + "\"\n"
+		set_icon_name("media-playback-start");
+
+		std::string tooltip_text =
+				"Short label: \"" + FICDecoder::DeriveShortLabelUTF8(label, service.label.short_label_mask) + "\"\n"
 				"Label charset: " + charset_name + "\n"
-				"SId: " + sid_string + (!service.IsPrimary() ? " (SCIdS: " + std::to_string(service.scids) + ")" : "") + "\n"
+				"SId: " + StringTools::IntToHex(service.sid, 4) + (!service.IsPrimary() ? " (SCIdS: " + std::to_string(service.scids) + ")" : "") + "\n"
 				"SubChId: " + std::to_string(service.audio_service.subchid) + "\n"
-				"Audio type: " + (service.audio_service.dab_plus ? "DAB+" : "DAB")
-		);
+				"Audio type: " + (service.audio_service.dab_plus ? "DAB+" : "DAB");
+		if(ensemble.inter_table_id != FIC_ENSEMBLE::inter_table_id_none) {
+			if(service.pty_static != LISTED_SERVICE::pty_none)
+				tooltip_text += "\n" "Programme type (static): " + FICDecoder::ConvertPTYToString(service.pty_static, ensemble.inter_table_id);
+			if(service.pty_dynamic != LISTED_SERVICE::pty_none)
+				tooltip_text += "\n" "Programme type (dynamic): " + FICDecoder::ConvertPTYToString(service.pty_dynamic, ensemble.inter_table_id);
+		}
+		frame_combo_services.set_tooltip_text(tooltip_text);
+
 		if(!service.subchannel.IsNone()) {
-			std::string tooltip_text;
+			tooltip_text = "";
 			if(!service.subchannel.pl.empty()) {
 				tooltip_text +=
 						"Sub-channel start: " + std::to_string(service.subchannel.start) + " CUs\n"
@@ -457,11 +473,15 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 			}
 			frame_label_format.set_tooltip_text(tooltip_text);
 		}
+
 		tglbtn_record.set_sensitive(true);
 	} else {
 		set_title("DABlin");
+		set_icon_name("media-playback-stop");
+
 		frame_combo_services.set_tooltip_text("");
 		frame_label_format.set_tooltip_text("");
+
 		tglbtn_record.set_sensitive(false);
 	}
 
@@ -542,6 +562,7 @@ void DABlinGTK::on_tglbtn_record() {
 			combo_channels.set_sensitive(false);	// parent frame already non-sensitive, if channels not available
 			combo_services.set_sensitive(false);
 
+			set_icon_name("media-record");
 			fprintf(stderr, "DABlinGTK: recording started into '%s'\n", new_rec_filename.c_str());
 
 			{
@@ -585,6 +606,7 @@ void DABlinGTK::on_tglbtn_record() {
 			fclose(rec_file);
 			rec_file = nullptr;
 
+			set_icon_name("media-playback-start");
 			fprintf(stderr, "DABlinGTK: recording stopped\n");
 			UpdateRecStatus();
 
@@ -641,11 +663,11 @@ void DABlinGTK::UpdateRecStatus() {
 	if(rec_file) {
 		tooltip_text =
 				"File name: \"" + rec_filename + "\"\n"
-				"Duration: " + MiscTools::MsToTimecode(rec_duration_ms);
+				"Duration: " + StringTools::MsToTimecode(rec_duration_ms);
 	} else {
 		// if prebuffer enabled
 		if(rec_prebuffer_filled_ms > 0)
-			tooltip_text = "Prebuffer: " + MiscTools::MsToTimecode(rec_prebuffer_filled_ms) + " / " + MiscTools::MsToTimecode(options.rec_prebuffer_size_s * 1000);
+			tooltip_text = "Prebuffer: " + StringTools::MsToTimecode(rec_prebuffer_filled_ms) + " / " + StringTools::MsToTimecode(options.rec_prebuffer_size_s * 1000);
 	}
 
 	tglbtn_record.set_tooltip_text(tooltip_text);
@@ -684,14 +706,9 @@ bool DABlinGTK::on_window_delete_event(GdkEventAny* /*any_event*/) {
 	}
 }
 
-void DABlinGTK::ConnectKeyPressEventHandler(Gtk::Widget& widget) {
-	widget.signal_key_press_event().connect(sigc::mem_fun(*this, &DABlinGTK::HandleKeyPressEvent));
-	widget.add_events(Gdk::KEY_PRESS_MASK);
-}
-
 bool DABlinGTK::HandleKeyPressEvent(GdkEventKey* key_event) {
-	// consider only events without Shift/Control/Alt
-	if((key_event->state & (Gdk::SHIFT_MASK | Gdk::CONTROL_MASK | Gdk::MOD1_MASK)) == 0) {
+	// events without Shift/Control/Alt/Win
+	if((key_event->state & (Gdk::SHIFT_MASK | Gdk::CONTROL_MASK | Gdk::MOD1_MASK | Gdk::SUPER_MASK)) == 0) {
 		switch(key_event->keyval) {
 		case GDK_KEY_m:
 		case GDK_KEY_M:
@@ -704,64 +721,121 @@ bool DABlinGTK::HandleKeyPressEvent(GdkEventKey* key_event) {
 			if(tglbtn_record.get_sensitive())
 				tglbtn_record.clicked();
 			return true;
-		// try to switch service
-		case GDK_KEY_1:
-		case GDK_KEY_KP_1:
-			TryServiceSwitch(0);
-			return true;
-		case GDK_KEY_2:
-		case GDK_KEY_KP_2:
-			TryServiceSwitch(1);
-			return true;
-		case GDK_KEY_3:
-		case GDK_KEY_KP_3:
-			TryServiceSwitch(2);
-			return true;
-		case GDK_KEY_4:
-		case GDK_KEY_KP_4:
-			TryServiceSwitch(3);
-			return true;
-		case GDK_KEY_5:
-		case GDK_KEY_KP_5:
-			TryServiceSwitch(4);
-			return true;
-		case GDK_KEY_6:
-		case GDK_KEY_KP_6:
-			TryServiceSwitch(5);
-			return true;
-		case GDK_KEY_7:
-		case GDK_KEY_KP_7:
-			TryServiceSwitch(6);
-			return true;
-		case GDK_KEY_8:
-		case GDK_KEY_KP_8:
-			TryServiceSwitch(7);
-			return true;
-		case GDK_KEY_9:
-		case GDK_KEY_KP_9:
-			TryServiceSwitch(8);
-			return true;
-		case GDK_KEY_0:
-		case GDK_KEY_KP_0:
-			TryServiceSwitch(9);
-			return true;
-		case GDK_KEY_minus:
-		case GDK_KEY_KP_Subtract:
-			TryServiceSwitch(combo_services.get_active_row_number() - 1);
-			return true;
-		case GDK_KEY_plus:
-		case GDK_KEY_KP_Add:
-			TryServiceSwitch(combo_services.get_active_row_number() + 1);
+		}
+
+		// try to switch service (1 to 10, -/+)
+		int new_index;
+		if(CheckForIndexKey(key_event, combo_services.get_active_row_number(), new_index)) {
+			TrySwitch(combo_services, combo_services_liststore, new_index);
 			return true;
 		}
 	}
+
+	// events without Shift/Control/Win, but with Alt
+	if((key_event->state & (Gdk::SHIFT_MASK | Gdk::CONTROL_MASK | Gdk::SUPER_MASK)) == 0 && (key_event->state & Gdk::MOD1_MASK)) {
+		// try to switch service (11 to 20)
+		int new_index;
+		if(CheckForIndexKey(key_event, -1, new_index)) {
+			TrySwitch(combo_services, combo_services_liststore, new_index + 10);
+			return true;
+		}
+	}
+
+	// events without Shift/Alt/Win, but with Control
+	if((key_event->state & (Gdk::SHIFT_MASK | Gdk::MOD1_MASK | Gdk::SUPER_MASK)) == 0 && (key_event->state & Gdk::CONTROL_MASK)) {
+		// try to switch channel (1 to 10, -/+)
+		int new_index;
+		if(CheckForIndexKey(key_event, combo_channels.get_active_row_number(), new_index)) {
+			TrySwitch(combo_channels, combo_channels_liststore, new_index);
+			return true;
+		}
+
+		switch(key_event->keyval) {
+		case GDK_KEY_c:
+		case GDK_KEY_C:
+			// copy DL to clipboard, if not empty
+			std::string dl = label_dl.get_label();
+			if(!dl.empty())
+				Gtk::Clipboard::get()->set_text(dl);
+			return true;
+		}
+	}
+
+	// events without Shift/Win, but with Control/Alt
+	if((key_event->state & (Gdk::SHIFT_MASK | Gdk::SUPER_MASK)) == 0 && (key_event->state & Gdk::CONTROL_MASK) && (key_event->state & Gdk::MOD1_MASK)) {
+		// try to switch channel (11 to 20)
+		int new_index;
+		if(CheckForIndexKey(key_event, -1, new_index)) {
+			TrySwitch(combo_channels, combo_channels_liststore, new_index + 10);
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool DABlinGTK::CheckForIndexKey(GdkEventKey* key_event, int old_index, int& new_index) {
+	switch(key_event->keyval) {
+	case GDK_KEY_1:
+	case GDK_KEY_KP_1:
+		new_index = 0;
+		return true;
+	case GDK_KEY_2:
+	case GDK_KEY_KP_2:
+		new_index = 1;
+		return true;
+	case GDK_KEY_3:
+	case GDK_KEY_KP_3:
+		new_index = 2;
+		return true;
+	case GDK_KEY_4:
+	case GDK_KEY_KP_4:
+		new_index = 3;
+		return true;
+	case GDK_KEY_5:
+	case GDK_KEY_KP_5:
+		new_index = 4;
+		return true;
+	case GDK_KEY_6:
+	case GDK_KEY_KP_6:
+		new_index = 5;
+		return true;
+	case GDK_KEY_7:
+	case GDK_KEY_KP_7:
+		new_index = 6;
+		return true;
+	case GDK_KEY_8:
+	case GDK_KEY_KP_8:
+		new_index = 7;
+		return true;
+	case GDK_KEY_9:
+	case GDK_KEY_KP_9:
+		new_index = 8;
+		return true;
+	case GDK_KEY_0:
+	case GDK_KEY_KP_0:
+		new_index = 9;
+		return true;
+	case GDK_KEY_minus:
+	case GDK_KEY_KP_Subtract:
+		if(old_index == -1)
+			break;
+		new_index = old_index - 1;
+		return true;
+	case GDK_KEY_plus:
+	case GDK_KEY_KP_Add:
+		if(old_index == -1)
+			break;
+		new_index = old_index + 1;
+		return true;
+	}
 	return false;
 }
 
-void DABlinGTK::TryServiceSwitch(int index) {
-	// switch service, if allowed and index valid
-	if(combo_services.is_sensitive() && index >= 0 && index < (signed) combo_services_liststore->children().size())
-		combo_services.set_active(index);
+void DABlinGTK::TrySwitch(Gtk::ComboBox& combo, Glib::RefPtr<Gtk::ListStore>& combo_liststore, int index) {
+	// switch, if allowed and index valid
+	if(combo.is_sensitive() && index >= 0 && index < (signed) combo_liststore->children().size())
+		combo.set_active(index);
 }
 
 bool DABlinGTK::HandleConfigureEvent(GdkEventConfigure* /*configure_event*/) {
@@ -791,19 +865,22 @@ void DABlinGTK::ETIChangeFormatEmitted() {
 void DABlinGTK::FICChangeEnsembleEmitted() {
 //	fprintf(stderr, "### FICChangeEnsembleEmitted\n");
 
-	FIC_ENSEMBLE new_ensemble = fic_change_ensemble.Pop();
-
-	char eid_string[7];
-	snprintf(eid_string, sizeof(eid_string), "0x%04X", new_ensemble.eid);
+	ensemble = fic_change_ensemble.Pop();
 
 	std::string charset_name;
-	Glib::ustring label = FICDecoder::ConvertLabelToUTF8(new_ensemble.label, &charset_name);
+	std::string label = FICDecoder::ConvertLabelToUTF8(ensemble.label, &charset_name);
+
+	std::string tooltip_text =
+			"Short label: \"" + FICDecoder::DeriveShortLabelUTF8(label, ensemble.label.short_label_mask) + "\"\n"
+			"Label charset: " + charset_name + "\n"
+			"EId: " + StringTools::IntToHex(ensemble.eid, 4);
+	if(ensemble.ecc != FIC_ENSEMBLE::ecc_none)
+		tooltip_text += "\n" "ECC: " + StringTools::IntToHex(ensemble.ecc, 2);
+	if(ensemble.inter_table_id != FIC_ENSEMBLE::inter_table_id_none)
+		tooltip_text += "\n" "International table ID: " + StringTools::IntToHex(ensemble.inter_table_id, 2) + " (" + FICDecoder::ConvertInterTableIDToString(ensemble.inter_table_id) + ")";
 
 	label_ensemble.set_label(label);
-	frame_label_ensemble.set_tooltip_text(
-			"Short label: \"" + DeriveShortLabel(label, new_ensemble.label.short_label_mask) + "\"\n"
-			"Label charset: " + charset_name + "\n"
-			"EId: " + eid_string);
+	frame_label_ensemble.set_tooltip_text(tooltip_text);
 }
 
 void DABlinGTK::FICChangeServiceEmitted() {
@@ -812,7 +889,7 @@ void DABlinGTK::FICChangeServiceEmitted() {
 	LISTED_SERVICE new_service = fic_change_service.Pop();
 
 	std::string label = FICDecoder::ConvertLabelToUTF8(new_service.label, nullptr);
-	Glib::ustring combo_label = label;
+	std::string combo_label = label;
 	if(new_service.multi_comps)
 		combo_label = (!new_service.IsPrimary() ? "» " : "") + combo_label + (new_service.IsPrimary() ? " »" : "");
 
@@ -902,7 +979,7 @@ void DABlinGTK::PADChangeDynamicLabelEmitted() {
 	// consider clear display command
 	if(dl.charset != -1) {
 		std::string charset_name;
-		Glib::ustring label = FICDecoder::ConvertTextToUTF8(&dl.raw[0], dl.raw.size(), dl.charset, false, &charset_name);
+		std::string label = CharsetTools::ConvertTextToUTF8(&dl.raw[0], dl.raw.size(), dl.charset, false, &charset_name);
 
 		// skip unsupported charsets
 		if(!charset_name.empty()) {
@@ -928,14 +1005,3 @@ void DABlinGTK::PADChangeSlideEmitted() {
 void DABlinGTK::PADLengthError(size_t /*announced_xpad_len*/, size_t /*xpad_len*/) {
 	fprintf(stderr, "\x1B[31m" "[X-PAD len]" "\x1B[0m" " ");
 }
-
-
-Glib::ustring DABlinGTK::DeriveShortLabel(Glib::ustring long_label, uint16_t short_label_mask) {
-	Glib::ustring short_label;
-
-	for(size_t i = 0; i < long_label.length(); i++)		// consider discarded trailing spaces
-		if(short_label_mask & (0x8000 >> i))
-			short_label += long_label[i];
-
-	return short_label;
-}
