@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2018 Stefan Pöschel
+    Copyright (C) 2015-2019 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,8 +25,9 @@ ETISource::ETISource(std::string filename, ETISourceObserver *observer) {
 	this->observer = observer;
 
 	input_file = nullptr;
-	eti_frame_count = 0;
-	eti_frame_total = 0;
+	eti_frames_count = 0;
+	eti_bytes_count = 0;
+	eti_bytes_total = 0;
 	eti_progress_next_ms = 0;
 
 	do_exit = false;
@@ -54,12 +55,12 @@ bool ETISource::OpenFile() {
 		}
 	}
 
-	// init total frames
-	return UpdateTotalFrames();
+	// init total bytes
+	return UpdateTotalBytes();
 }
 
-bool ETISource::UpdateTotalFrames() {
-	// if file size available, calc total frame count
+bool ETISource::UpdateTotalBytes() {
+	// if file size available, get total bytes count
 	off_t old_offset = ftello(input_file);
 	if(old_offset == -1) {
 		// ignore non-seekable files (like usually stdin)
@@ -85,7 +86,7 @@ bool ETISource::UpdateTotalFrames() {
 		return false;
 	}
 
-	eti_frame_total = (len / sizeof(eti_frame)) - 1;
+	eti_bytes_total = len;
 	return true;
 }
 
@@ -115,6 +116,7 @@ int ETISource::Main() {
 	fd_set fds;
 	timeval select_timeval;
 	size_t filled = 0;
+	size_t sync_skipped = 0;
 
 	while(!do_exit) {
 		FD_ZERO(&fds);
@@ -135,11 +137,22 @@ int ETISource::Main() {
 
 		size_t bytes = fread(eti_frame + filled, 1, sizeof(eti_frame) - filled, input_file);
 
-		if(bytes > 0)
+		if(bytes > 0) {
+			eti_bytes_count += bytes;
 			filled += bytes;
+		}
 		if(bytes == 0) {
 			if(feof(input_file)) {
+				size_t sync_skipped_eof = sync_skipped + filled;
+				if(sync_skipped_eof)
+					fprintf(stderr, "ETISource: skipping %zu bytes at EOF\n", sync_skipped_eof);
 				fprintf(stderr, "ETISource: EOF reached!\n");
+
+				// if present, update progress
+				if(eti_bytes_total) {
+					if(!UpdateProgress())
+						return 1;
+				}
 				break;
 			}
 			perror("ETISource: error while fread");
@@ -149,28 +162,57 @@ int ETISource::Main() {
 		if(filled < sizeof(eti_frame))
 			continue;
 
-		// if present, update progress every 500ms or at file end
-		if(eti_frame_total && (eti_frame_count * 24 >= eti_progress_next_ms || eti_frame_count == eti_frame_total)) {
-			// update total frames
-			if(!UpdateTotalFrames())
-				return 1;
-
-			ETI_PROGRESS progress;
-			progress.value = (double) eti_frame_count / (double) eti_frame_total;
-			progress.text = MiscTools::MsToTimecode(eti_frame_count * 24) + " / " + MiscTools::MsToTimecode(eti_frame_total * 24);
-			observer->ETIUpdateProgress(progress);
-
-			eti_progress_next_ms += 500;
+		// check for frame sync
+		size_t offset;
+		for(offset = 0; offset < sizeof(eti_frame) - 3; offset++) {
+			uint32_t fsync = eti_frame[offset+1] << 16 | eti_frame[offset+2] << 8 | eti_frame[offset+3];
+			if(fsync == 0x073AB6 || fsync == 0xF8C549)
+				break;
 		}
 
-		observer->ETIProcessFrame(eti_frame);
-		eti_frame_count++;
-		filled = 0;
+		if(offset) {	// buffer not (yet) synced
+			// discard buffer start
+			memmove(eti_frame, eti_frame + offset, sizeof(eti_frame) - offset);
+			filled -= offset;
+			sync_skipped += offset;
+		} else {		// buffer synced
+			if(sync_skipped) {
+				fprintf(stderr, "ETISource: skipping %zu bytes for sync\n", sync_skipped);
+				sync_skipped = 0;
+			}
+
+			// if present, update progress every 500ms
+			if(eti_bytes_total && eti_frames_count * 24 >= eti_progress_next_ms) {
+				if(!UpdateProgress())
+					return 1;
+				eti_progress_next_ms += 500;
+			}
+
+			observer->ETIProcessFrame(eti_frame);
+			eti_frames_count++;
+			filled = 0;
+		}
 	}
 
 	return 0;
 }
 
+bool ETISource::UpdateProgress() {
+	// update total bytes
+	if(!UpdateTotalBytes())
+		return false;
+
+	size_t eti_bytes_left = eti_bytes_total - eti_bytes_count;
+	size_t eti_frames_left = eti_bytes_left / sizeof(eti_frame);
+	size_t eti_frames_total = eti_frames_count + eti_frames_left;
+
+	ETI_PROGRESS progress;
+	progress.value = (double) eti_frames_count / (double) eti_frames_total;
+	progress.text = StringTools::MsToTimecode(eti_frames_count * 24) + " / " + StringTools::MsToTimecode(eti_frames_total * 24);
+	observer->ETIUpdateProgress(progress);
+	return true;
+}
+
 
 // --- DABLiveETISource -----------------------------------------------------------------
 const std::string DABLiveETISource::TYPE_DAB2ETI = "dab2eti";
